/**
 * @file prt_deque.h
 * @brief double-ended queue
 * @author Jakub Kurzak
 *
 * Thread-safe and size-aware.
 * Built on top of icl_list.
 * Uses Pthread spinlocks.
 *
 * PULSAR Runtime
 * http://icl.eecs.utk.edu/pulsar/
 * Copyright (C) 2012-2013 University of Tennessee.
 */

#ifndef ICL_DEQUE_H
#define ICL_DEQUE_H

#include <limits.h>
#include <stdlib.h>
#include <pthread.h>
#include "icl_list.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef icl_list_t icl_node_t;
typedef struct icl_deque_s {
    pthread_spinlock_t spinlock;
    icl_list_t *list;
    int size;
} icl_deque_t;

icl_deque_t* icl_deque_new();
int icl_deque_destroy(icl_deque_t *deque, void (*free_func)(void*));
icl_node_t* icl_deque_first(icl_deque_t *deque);
icl_node_t* icl_deque_next(icl_deque_t *deque, icl_node_t *node);
icl_node_t* icl_deque_append(icl_deque_t *deque, void *data);
icl_node_t* icl_deque_prepend(icl_deque_t *deque, void *data);
int icl_deque_delete(icl_deque_t *deque, icl_node_t *node, void (*free_func)(void*));
int icl_deque_size(icl_deque_t *deque);

#ifdef __cplusplus
}
#endif

#endif /* ICL_DEQUE_H */
