/**
 * @file icl_hash.h
 * @brief dependency free hash table
 * @author Keith Seymour
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.eecs.utk.edu/pulsar/
 * Copyright (C) 2012-2013 University of Tennessee.
 */

#ifndef ICL_HASH_H
#define ICL_HASH_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct icl_entry_s {
    void *key;
    void *data;
    struct icl_entry_s *next;
} icl_entry_t;

typedef struct icl_hash_s {
    int nbuckets;
    int nentries;
    icl_entry_t **buckets;
    unsigned int (*hash_function)(void*);
    int (*hash_key_compare)(void*, void*);
} icl_hash_t;

icl_hash_t* icl_hash_create(
    int nbuckets,
    unsigned int (*hash_function)(void*),
    int (*hash_key_compare)(void*, void*));
void* icl_hash_find(icl_hash_t*, void*);
icl_entry_t* icl_hash_insert(icl_hash_t *, void*, void *);
icl_entry_t* icl_hash_update_insert(icl_hash_t*, void*, void*, void**);
int icl_hash_destroy(icl_hash_t*, void (*)(void*), void (*)(void*));
int icl_hash_dump(FILE*, icl_hash_t*);
int icl_hash_delete(icl_hash_t*, void*, void (*)(void*), void (*)(void*));

#define icl_hash_foreach(ht, tmpint, tmpent, kp, dp)    \
    for (tmpint=0;tmpint<ht->nbuckets; tmpint++)        \
        for (tmpent=ht->buckets[tmpint];                \
             tmpent!=NULL&&((kp=tmpent->key)!=NULL)&&((dp=tmpent->data)!=NULL); \
             tmpent=tmpent->next)

#ifdef __cplusplus
}
#endif

#endif /* ICL_HASH_H */
