/**
 * @file icl_list.h
 * @brief dependency free linked list
 * @author Keith Seymour
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.eecs.utk.edu/pulsar/
 * Copyright (C) 2012-2013 University of Tennessee.
 */

#ifndef ICL_LIST_H
#define ICL_LIST_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct icl_list_s {
    void *data;
    struct icl_list_s *flink;
    struct icl_list_s *blink;
} icl_list_t;

icl_list_t* icl_list_new();
icl_list_t* icl_list_insert(icl_list_t *, icl_list_t *, void *);
icl_list_t* icl_list_search(icl_list_t *, void *, int (*)(void*, void*));
icl_list_t* icl_list_isort(icl_list_t *head, void *data, int (*)(void*, void*));
icl_list_t* icl_list_first(icl_list_t *);
icl_list_t* icl_list_last(icl_list_t *);
icl_list_t* icl_list_next(icl_list_t *, icl_list_t *);
icl_list_t* icl_list_prev(icl_list_t *, icl_list_t *);
icl_list_t* icl_list_concat(icl_list_t *, icl_list_t *);
icl_list_t* icl_list_prepend(icl_list_t *, void *);
icl_list_t* icl_list_append(icl_list_t *, void *);

int icl_list_delete(icl_list_t *, icl_list_t *, void (*)(void *));
int icl_list_destroy(icl_list_t *, void (*)(void*));
int icl_list_size(icl_list_t *);

#define icl_list_foreach(list, ptr) \
    for (ptr = icl_list_first(list); ptr != NULL; ptr = icl_list_next(list, ptr))

#ifdef __cplusplus
}
#endif

#endif /* ICL_LIST_H */
