/**
 * @file prt_assert.c
 * @brief PRT exception handling
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.eecs.utk.edu/pulsar/
 * Copyright (C) 2012-2013 University of Tennessee.
 */

#include "prt_assert.h"

////////////////////////////////////////////////////////////////////////////////
/**
 * @brief Check an assertion and exit on error.
 * Print an error message.
 *
 * @param cond condition
 * @param msg error message
 * @param line line number
 * @param file source file name
 */
void prt_assert_line_file(int cond, char *msg, int line, char *file)
{
    if (!cond)
        prt_error_line_file(msg, line, file);
}

////////////////////////////////////////////////////////////////////////////////
/**
 * @brief Print an error message and exit.
 *
 * @param msg error message
 * @param line line number
 * @param file source file name
 */
void prt_error_line_file(char *msg, int line, char *file)
{
    fprintf(stderr, "\nPRT ERROR: %s (in %s at line %d)\n\n", msg, file, line);
    fflush(stderr);
    MPI_Abort(MPI_COMM_WORLD, 0);
}

////////////////////////////////////////////////////////////////////////////////
/**
 * @brief Print a warning and continue.
 *
 * @param msg warning message
 * @param line line number
 * @param file source file name
 */
void prt_warning_line_file(char *msg, int line, char *file)
{
    fprintf(stderr, "\nPRT WARNING: %s (in %s at line %d)\n\n", msg, file, line);
    fflush(stderr);
}
