/**
 * @file prt_config.c
 * @brief PRT configuration
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.eecs.utk.edu/pulsar/
 * Copyright (C) 2012-2013 University of Tennessee.
 */

#include "prt_config.h"

////////////////////////////////////////////////////////////////////////////////
/**
 * @brief config object constructor
 *
 * @return new config object with default values
 */
prt_config_t* prt_config_new()
{
    // Allocate the config object.
    prt_config_t *config = (prt_config_t*)malloc(sizeof(prt_config_t));
    prt_assert(config != NULL, "malloc failed");

    // Set the defaults.
    config->vdp_scheduling = PRT_VDP_SCHEDULING_AGGRESSIVE;
    config->vdp_scheduling = PRT_SVG_TRACING_OFF;

    // Return the config object.
    return config;
}

////////////////////////////////////////////////////////////////////////////////
/**
 * @brief config object destructor
 *
 * @param vdp
 */
void prt_config_delete(prt_config_t *config)
{
    // Free the config object.
    free(config);
}
