/**
 * @file prt_config.h
 * @brief PRT configuration
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.eecs.utk.edu/pulsar/
 * Copyright (C) 2012-2013 University of Tennessee.
 */

#ifndef PRT_CONFIG_H
#define PRT_CONFIG_H

#include "prt.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief PRT configuration parameters
 */
typedef enum prt_config_param_e {
    PRT_VDP_SCHEDULING,
    PRT_SVG_TRACING
} prt_config_param_t;

/**
 * @brief values of PRT configuration parameters
 */
typedef enum prt_config_value_e {
    PRT_VDP_SCHEDULING_AGGRESSIVE,  // Run each VDP as long as possible.
    PRT_VDP_SCHEDULING_LAZY,        // Fire each VDP once and move on.
    PRT_SVG_TRACING_ON,             // produce an SVG trace file
    PRT_SVG_TRACING_OFF             // skip the SVG trace file
} prt_config_value_t;

/**
 * @brief PRT configuration
 */
typedef struct prt_config_s {
    int vdp_scheduling;         // VDP scheduling mode
    int svg_tracing;            // SVG tracing on/off
} prt_config_t;

prt_config_t* prt_config_new();
void prt_config_delete(prt_config_t *config);

#ifdef __cplusplus
}
#endif

#endif /* PRT_CONFIG_H */
