/**
 * @file prt_packet.c
 * @brief PRT data packet implementation
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.eecs.utk.edu/pulsar/
 * Copyright (C) 2012-2013 University of Tennessee.
 */

#include "prt_packet.h"

////////////////////////////////////////////////////////////////////////////////
/**
 * @ingroup API
 * @brief packet constructor
 * Sets the number of references to one.
 *
 * @param data_size size of the packet in bytes
 *
 * @return new packet
 */
prt_packet_t *prt_packet_new(size_t data_size)
{
    prt_packet_t *packet = (prt_packet_t*)malloc(sizeof(prt_packet_t));
    prt_assert(packet != NULL, "malloc failed");

    packet->data = malloc(data_size);
    prt_assert(packet->data != NULL, "malloc failed");

    packet->num_refs = 1;
    return packet;
}

////////////////////////////////////////////////////////////////////////////////
/**
 * @ingroup API
 * @brief Release a packet.
 * Decrements the number of active references.
 * Destroys the packet when the last reference is removed.
 *
 * @param packet
 */
void prt_packet_release(prt_packet_t *packet)
{
    int num_refs = __sync_sub_and_fetch(&packet->num_refs, 1);
    prt_assert(num_refs >= 0, "negative number of data references");
    if (num_refs == 0) {
        free(packet->data);
        free(packet);
    }
}
