/**
 * @file prt_proxy.h
 * @brief PRT communication proxy
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.eecs.utk.edu/pulsar/
 * Copyright (C) 2012-2013 University of Tennessee.
 */

#ifndef PRT_PROXY_H
#define PRT_PROXY_H

#include "prt.h"

#ifdef __cplusplus
extern "C" {
#endif

struct prt_vsa_s;
struct prt_channel_s;
struct prt_request_s;

/**
 * @brief maximum tags per node
 * Size of the proxy's hash table of tags.
 * Should be a prime number.
 */
#define PRT_PROXY_MAX_TAGS_PER_NODE 10003
#define PRT_PROXY_MAX_SENDS_PER_THREAD 1
#define PRT_PROXY_MAX_RECVS_PER_THREAD 1

/**
 * @brief VSA's communication proxy
 * Serves communication requests from the worker threads.
 * Contains a list of recv requests from each worker thread.
 * Contains a list of send requests from each worker thread.
 */
typedef struct prt_proxy_s {
    struct prt_vsa_s *vsa;          // back pointer to the parent VSA.
    int num_threads;                // number of threads
    icl_hash_t *tags_hash;          // channel lookup by rank-tag
    icl_deque_t **sends_requested;  // sends requested by channels (per thread)
    icl_list_t **sends_posted;      // sends posted, not completed (per thread)
    icl_list_t *recvs_posted;       // recvs posted, not completed (all threads)
    size_t max_packet_size;         // maximum size of a packet in bytes
} prt_proxy_t;

prt_proxy_t* prt_proxy_new(int num_threads);
void prt_proxy_delete(prt_proxy_t *proxy);
void prt_proxy_max_packet_size(prt_proxy_t *proxy, struct prt_channel_s *channel);
void prt_proxy_send(prt_proxy_t *proxy, int thread_rank, struct prt_channel_s *channel);
void prt_proxy_recv(prt_proxy_t *proxy, struct prt_request_s *request);
void prt_proxy_run(prt_proxy_t *proxy);

#ifdef __cplusplus
}
#endif

#endif /* PRT_PROXY_H */
