/**
 * @file prt_request.h
 * @brief PRT communication request
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.eecs.utk.edu/pulsar/
 * Copyright (C) 2012-2013 University of Tennessee.
 */

#ifndef PRT_REQUEST_H
#define PRT_REQUEST_H

#include "prt.h"

#ifdef __cplusplus
extern "C" {
#endif

struct prt_packet_s;

/**
 * @brief VSA proxy's communication request
 * Contains basic information about the communication request.
 * Contains the MPI request.
 */
typedef struct prt_request_s {
    struct prt_packet_s *packet;
    int count;
    MPI_Datatype datatype;
    int peer;
    int tag;
    MPI_Request request;
    MPI_Status status;
} prt_request_t;

prt_request_t *prt_request_new(
    struct prt_packet_s *packet, int count,
    MPI_Datatype datatype, int peer, int tag);
void prt_request_destroy(prt_request_t *request);
void prt_request_send(prt_request_t *request);
void prt_request_recv(prt_request_t *request);
int prt_request_test(prt_request_t *request);
void prt_request_cancel(prt_request_t *request);

#ifdef __cplusplus
}
#endif

#endif /* PRT_REQUEST_H */
