/**
 * @file prt_thread.h
 * @brief PRT thread
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.eecs.utk.edu/pulsar/
 * Copyright (C) 2012-2013 University of Tennessee.
 */

#ifndef PRT_THREAD_H
#define PRT_THREAD_H

#include "prt.h"

#ifdef __cplusplus
extern "C" {
#endif

struct prt_vsa_s;

/**
 * @brief VSA's worker thread
 * Owns a number of VDPs.
 * Knows the communication proxy.
 */
typedef struct prt_thread_s {
    struct prt_vsa_s *vsa;  // back link to the parent VSA
    int rank;               // local thread number
    int core;               // global core number
    pthread_t id;           // local pthread ID
    icl_list_t *vdps;       // list of VDPs
    int finished;           // completion flag
} prt_thread_t;

prt_thread_t* prt_thread_new(int rank, int core);
void prt_thread_delete(prt_thread_t *thread);
void* prt_thread_run(void *thrd);

#ifdef __cplusplus
}
#endif

#endif /* PRT_THREAD_H */
