/**
 * @file prt_tuple.h
 * @brief PRT tuple
 * @author Jakub Kurzak
 *
 * Tuples uniquely identify VDPs in a VSA.
 * Tuple is an array of integers terminated with INT_MAX.
 * For all practical purposes a tuple behaves like a string.
 *
 * PULSAR Runtime
 * http://icl.eecs.utk.edu/pulsar/
 * Copyright (C) 2012-2013 University of Tennessee.
 */

#ifndef PRT_TUPLE_H
#define PRT_TUPLE_H

#include "prt.h"

#ifdef __cplusplus
extern "C" {
#endif

int* prt_tuple_new(int len, ...);
int prt_tuple_len(int *tuple);
int* prt_tuple_cat(int *first_tuple, ...);
void prt_tuple_delete(int *tuple);
int* prt_tuple_copy(int *in_tuple);
int prt_tuple_compare(void *tuple_a, void *tuple_b);
int prt_tuple_equal(void *tuple_a, void *tuple_b);
void prt_tuple_print(int *tuple);
unsigned int prt_tuple_hash(void *tuple);

#define prt_tuple_new1(a) prt_tuple_new(1,a)
#define prt_tuple_new2(a,b) prt_tuple_new(2,a,b)
#define prt_tuple_new3(a,b,c) prt_tuple_new(3,a,b,c)
#define prt_tuple_new4(a,b,c,d) prt_tuple_new(4,a,b,c,d)
#define prt_tuple_new5(a,b,c,d,e) prt_tuple_new(5,a,b,c,d,e)
#define prt_tuple_new6(a,b,c,d,e,f) prt_tuple_new(6,a,b,c,d,e,f)

#ifdef __cplusplus
}
#endif

#endif /* PRT_TUPLE_H */
