
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>

///////////////////////////////////////////////////////////////////////////////
/**
 * @brief Get time.
 * Return current time in seconds as a double-precision floating-point number.
 */
double pulsar_time_of_day()
{
    struct timeval  time_val;
    struct timezone time_zone;
    gettimeofday(&time_val, &time_zone);
    return (double)(time_val.tv_sec) + (double)(time_val.tv_usec) / 1000000.0;
}

////////////////////////////////////////////////////////////////////////////////
/**
 * @brief Initialize a tile with random numbers.
 * Call DLARNV with the seed derived from the tile's position.
 *
 * @param tile
 * @param m vertical position
 * @param n horizontal position
 * @param nb tile size
 */
void pulsar_tile_init(double *tile, int m, int n, int nb)
{
    int iseed[4];
    iseed[0] = m & 0x0FFF;
    iseed[1] = n & 0x0FFF;
    iseed[2] = ((m >> 12) + (n >> 12)) & 0x0FFF;
    iseed[3] = 1;
    LAPACKE_dlarnv_work(1, iseed, nb*nb, tile);
}

////////////////////////////////////////////////////////////////////////////////
/**
 * @brief Print corners of a tile.
 * Print 2x2 corners of a tile.
 *
 * @param tile
 * @param nb tile size
 */
void pulsar_tile_print(double *tile, int nb)
{
    int i, j;
    printf("\n");
    for (j = 0; j < nb; j++) {
        if (j == 2) {
            j = nb-2;
            printf("  ...   ...   ...   ...   ... \n");
        }
        for (i = 0; i < nb; i++) {
            if (i == 2) {
                i = nb-2;
                printf("  ...");
            }
            printf("%6.2lf", tile[i*nb+j]);
        }
        printf("\n");
    }
}
